@@ -34,21 +34,21 @@ _FROM = 1
 _TO = 2
 
 
-def _process_class(cls):
+def _process_class(cls, **options):
 
     if not is_dataclass(cls):
         raise TypeError("must be called with a dataclass type")
 
-    _process_class_internal(cls)
+    _process_class_internal(cls, options=options)
 
     def from_dict(cls, *args, **kwargs):
-        inst = cls._fastclasses_json_from_dict(*args, **kwargs)
-        cls.from_dict = cls._fastclasses_json_from_dict
+        inst = getattr(cls, _from_dict_func(options))(*args, **kwargs)
+        cls.from_dict = getattr(cls, _from_dict_func(options))
         return inst
 
     def to_dict(self, *args, **kwargs):
-        d = self._fastclasses_json_to_dict(*args, **kwargs)
-        cls.to_dict = cls._fastclasses_json_to_dict
+        d = getattr(self, _to_dict_func(options))(*args, **kwargs)
+        cls.to_dict = getattr(cls, _to_dict_func(options))
         return d
 
     cls.from_dict = classmethod(from_dict)
@@ -69,29 +69,45 @@ def _process_class(cls):
     return cls
 
 
-def _process_class_internal(cls):
+def _from_dict_func(options):
+    if options:
+        return '_fastclasses_json_from_dict_%x' % _hash_options(options)
+    return '_fastclasses_json_from_dict'
+
+
+def _to_dict_func(options):
+    if options:
+        return '_fastclasses_json_to_dict_%x' % _hash_options(options)
+    return '_fastclasses_json_to_dict'
+
+
+def _hash_options(options):
+    return abs(hash(tuple(options.items())))
+
+
+def _process_class_internal(cls, options):
 
     # Delay the building of our from_dict method until it is first called.
     # This allows the compilation to reference classes defined later in
     # the module.
     def _temp_from_dict(cls, *args, **kwarg):
-        _replace_from_dict(cls, '_fastclasses_json_from_dict')
-        return cls._fastclasses_json_from_dict(*args, **kwarg)
+        _replace_from_dict(cls, options, _from_dict_func(options))
+        return getattr(cls, _from_dict_func(options))(*args, **kwarg)
 
-    cls._fastclasses_json_from_dict = classmethod(_temp_from_dict)
+    setattr(cls, _from_dict_func(options), classmethod(_temp_from_dict))
 
     def _temp_to_dict(self, *args, **kwargs):
-        _replace_to_dict(cls, '_fastclasses_json_to_dict')
-        return self._fastclasses_json_to_dict(*args, **kwargs)
+        _replace_to_dict(cls, options, _to_dict_func(options))
+        return getattr(self, _to_dict_func(options))(*args, **kwargs)
 
-    cls._fastclasses_json_to_dict = _temp_to_dict
+    setattr(cls, _to_dict_func(options), _temp_to_dict)
 
     return cls
 
 
-def _replace_from_dict(cls, from_dict='from_dict'):
+def _replace_from_dict(cls, options, from_dict='from_dict'):
 
-    from_dict_src = _from_dict_source(cls)
+    from_dict_src = _from_dict_source(cls, options)
     from_dict_module = compile(
         from_dict_src, '<fastclass_generated_code>', 'exec'
     )
@@ -121,9 +137,9 @@ def _replace_from_dict(cls, from_dict='from_dict'):
     setattr(cls, from_dict, classmethod(from_dict_func))
 
 
-def _replace_to_dict(cls, to_dict='to_dict'):
+def _replace_to_dict(cls, options, to_dict='to_dict'):
 
-    to_dict_src = _to_dict_source(cls)
+    to_dict_src = _to_dict_source(cls, options)
     to_dict_module = compile(
         to_dict_src, '<fastclass_generated_code>', 'exec'
     )
@@ -148,7 +164,7 @@ def _replace_to_dict(cls, to_dict='to_dict'):
     setattr(cls, to_dict, to_dict_func)
 
 
-def _from_dict_source(cls):
+def _from_dict_source(cls, options=None):
 
     lines = [
         'def from_dict(cls, o, *, infer_missing):',
@@ -165,14 +181,7 @@ def _from_dict_source(cls):
 
         field = fields_by_name[name]
 
-        input_name = name
-        if has_meta(field, 'field_name'):
-            input_name = field.metadata['fastclasses_json']['field_name']
-            if not isinstance(input_name, str):
-                raise TypeError(
-                    "fastclasses_json, field_name must be str: "
-                    f"{cls.__name__}.{name}"
-                )
+        input_name = deduce_serialised_name(name, options, field, cls)
 
         has_default = (
             field.default is not MISSING
@@ -183,7 +192,7 @@ def _from_dict_source(cls):
 
         access = f'o.get({input_name!r})'
 
-        transform = expr_builder_from(field_type)
+        transform = expr_builder_from(field_type, options)
         if has_meta(field, 'decoder'):
             transform = decoder_expr(name)
 
@@ -209,7 +218,7 @@ def _from_dict_source(cls):
     return '\n'.join(lines)
 
 
-def _to_dict_source(cls):
+def _to_dict_source(cls, options=None):
 
     lines = [
         'def to_dict(self):',
@@ -225,7 +234,7 @@ def _to_dict_source(cls):
 
         access = f'self.{name}'
 
-        transform = expr_builder_to(field_type)
+        transform = expr_builder_to(field_type, options)
 
         # custom encoder and decoder routines
         field = fields_by_name[name]
@@ -233,22 +242,15 @@ def _to_dict_source(cls):
             transform = encoder_expr(name)
 
         # custom mapping of dataclass fieldnames to json field names
-        output_name = name
-        if has_meta(field, 'field_name'):
-            output_name = field.metadata['fastclasses_json']['field_name']
-            if not isinstance(output_name, str):
-                raise TypeError(
-                    "fastclasses_json, field_name must be str: "
-                    f"{cls.__name__}.{name}"
-                )
+        output_name = deduce_serialised_name(name, options, field, cls)
 
         if transform('x') != 'x':
             # since we have an is not none check, elide the first level
             # of optional
             if (typing_get_origin(field_type) == typing.Union
                     # This is a bit yuk. Premature optimization 🙄
-                    and transform('x') == expr_builder_to(field_type)('x')):
-                transform = expr_builder_to(typing_get_args(field_type)[0])
+                    and transform('x') == expr_builder_to(field_type, options)('x')):
+                transform = expr_builder_to(typing_get_args(field_type)[0], options)
             lines.append(f'    value = {access}')
             lines.append(f'    if value is not None:')  # noqa: F541
             lines.append(f'        value = ' + transform('value'))  # noqa: E501,F541
@@ -264,6 +266,25 @@ def _to_dict_source(cls):
     return '\n'.join(lines)
 
 
+def deduce_serialised_name(name, options, field, cls):
+    serialised_name = name
+    if options and options.get('field_name_transform'):
+        serialised_name = options['field_name_transform'](name)
+        if not isinstance(serialised_name, str):
+            raise TypeError(
+                "fastclasses_json, result of field_name_transform must be str: "
+                f"{options['field_name_transform'].__name__}"
+            )
+    if has_meta(field, 'field_name'):
+        serialised_name = field.metadata['fastclasses_json']['field_name']
+        if not isinstance(serialised_name, str):
+            raise TypeError(
+                "fastclasses_json, field_name must be str: "
+                f"{cls.__name__}.{name}"
+            )
+    return serialised_name
+
+
 def encoders(cls):
     result = {}
     for field in dataclass_fields(cls):
@@ -298,15 +319,15 @@ def decoder_expr(name):
     return lambda expr: f'globals()["{name}#decoder"]({expr})'
 
 
-def expr_builder_from(t: type, depth=0):
-    return expr_builder(t, depth, direction=_FROM)
+def expr_builder_from(t: type, options, depth=0):
+    return expr_builder(t, options, depth, direction=_FROM)
 
 
-def expr_builder_to(t: type, depth=0):
-    return expr_builder(t, depth, direction=_TO)
+def expr_builder_to(t: type, options, depth=0):
+    return expr_builder(t, options, depth, direction=_TO)
 
 
-def expr_builder(t: type, depth=0, direction=_FROM):
+def expr_builder(t: type, options=None, depth=0, direction=_FROM):
     def identity(expr):
         return expr
 
@@ -316,7 +337,7 @@ def expr_builder(t: type, depth=0, direction=_FROM):
 
     if origin == typing.Union:
         type_arg = typing_get_args(t)[0]
-        inner = expr_builder(type_arg, depth + 1, direction)
+        inner = expr_builder(type_arg, options, depth + 1, direction)
 
         def f(expr):
             t0 = f'__{depth}'
@@ -330,7 +351,7 @@ def expr_builder(t: type, depth=0, direction=_FROM):
         type_args = typing_get_args(t)
         # Tuple[A, ...] means an any-length tuple of all As
         if type_args[1:] == (Ellipsis,):
-            inner = expr_builder(type_args[0], depth + 1, direction)
+            inner = expr_builder(type_args[0], options, depth + 1, direction)
 
             def f(expr):
                 t0 = f'__{depth}'
@@ -338,7 +359,7 @@ def expr_builder(t: type, depth=0, direction=_FROM):
             return f
         else:
             inners = [
-                expr_builder(type_arg, depth + 1, direction)
+                expr_builder(type_arg, options, depth + 1, direction)
                 for type_arg in type_args
             ]
 
@@ -364,7 +385,7 @@ def expr_builder(t: type, depth=0, direction=_FROM):
           and issubclass_safe(list, origin)
           and typing_get_args(t)):
         type_arg = typing_get_args(t)[0]
-        inner = expr_builder(type_arg, depth + 1, direction)
+        inner = expr_builder(type_arg, options, depth + 1, direction)
 
         def f(expr):
             t0 = f'__{depth}'
@@ -380,9 +401,9 @@ def expr_builder(t: type, depth=0, direction=_FROM):
             warnings.warn(f'to_json will not work for dict with key: {t}')
             return identity
 
-        inner = expr_builder(value_type, depth + 1, direction)
+        inner = expr_builder(value_type, options, depth + 1, direction)
 
-        key_func = expr_builder(key_type, depth + 1, direction)
+        key_func = expr_builder(key_type, options, depth + 1, direction)
         if direction == _FROM:
             if key_type is str:
                 pass
@@ -421,15 +442,15 @@ def expr_builder(t: type, depth=0, direction=_FROM):
     if is_dataclass(t):
         # Give indirectly referenced dataclasses a to_dict method without
         # trashing their public API
-        if not hasattr(t, '_fastclasses_json_from_dict'):
-            _process_class_internal(t)
+        if not hasattr(t, _from_dict_func(options)):
+            _process_class_internal(t, options)
 
         if direction == _FROM:
             def f(expr):
-                return f'{t.__name__}._fastclasses_json_from_dict({expr})'
+                return f'{t.__name__}.{_from_dict_func(options)}({expr})'
             return f
         else:
-            return lambda expr: f'({expr})._fastclasses_json_to_dict()'
+            return lambda expr: f'({expr}).{_to_dict_func(options)}()'
     elif issubclass_safe(t, Enum):
         if direction == _FROM:
             return lambda expr: f'{t.__name__}({expr})'
